<?php

/**
 * Methods for cURL request handling.
 */
class CURL {

    /**
     * The cURL request object.
     */
    private $request;

    /**
     * CURL class constructor.
     *
     * @throws Exception if an error occurs initializating.
     */
    public function __construct() {
        $this->request = curl_init();

        $this->throwExceptionIfError($this->request);
    }

    /**
     * Configure the cURL request.
     *
     * @param $url The target url.
     * @param $urlParameters The array of parameters, with 'key' => 'value' format.
     * @param $method 'GET' or 'POST'; 'GET' by default.
     * @param $moreOptions Any other options to add to the cURL request. By default,
     *    'CURLOPT_FOLLOWLOCATION' (follow 3XX redirects) and 'CURLOPT_RETURNTRANSFER'
     *    (return HTTP response as a value, instead of outputting directly) are set.
     * @throws Exception if an error occurs configurating.
     */
    public function configure($url, $urlParameters = [], $method = 'GET',
        $moreOptions = [CURLOPT_FOLLOWLOCATION => true, CURLOPT_RETURNTRANSFER => true]) {

        curl_reset($this->request);

        switch ($method) {
            case 'GET':
                $options = [CURLOPT_URL => $url . $this->stringifyParameters($urlParameters)];
                break;

            case 'POST':
                $options = [
                    CURLOPT_URL => $url,
                    CURLOPT_POST => true,
                    CURLOPT_POSTFIELDS => $this->stringifyParameters($urlParameters) 
                ];

                break;

            default:
                throw new Exception('Method must be "GET" or "POST".');
                break;
        }

        $options = $options + $moreOptions; 

        foreach ($options as $option => $value) {
            $configured = curl_setopt($this->request, $option, $value);

            $this->throwExceptionIfError($configured);
        }
    }

    /**
     * Executes the cURL request for the options configured.
     *
     * @return The return value of curl_exec(). If CURLOPT_RETURNTRANSFER was configured,
     *     the return value will be the HTTP response. In other case, a true value (or false
     *     if some error has occured).
     * @throws Exception if an error occurs executing.
     */
    public function execute() {
        $result = curl_exec($this->request);

        $this->throwExceptionIfError($result);

        return $result;
    }

    /**
     * Closes cURL session.
     */
    public function close() {
        curl_close($this->request);
    }

    /**
     * Checks if a curl_* function returns success or failuer, throwing an exception
     * with the cURL error message if it has failed.
     *
     * @param $success IF the curl function has succeeded or not.
     * @throws Exception if the curl function has failed.
     */
    protected function throwExceptionIfError($success) {
        if (!$success) {
            throw new Exception(curl_error($this->request));
        }
    }

    /**
     * Creates a string of GET parameters.
     *
     * @param $parameters Parameter array.
     * @return Parameters in string format: '?key1=value1&key2=value2'
     */
    protected function stringifyParameters($parameters) {
        $parameterString = '?';

        foreach ($parameters as $key => $value) {
            $key = urlencode($key);
            $value = urlencode($value);

            $parameterString .= "$key=$value&";
        }

        rtrim($parameterString, '&');

        return $parameterString;
    }
}
